from decimal import Decimal, InvalidOperation
from typing import Optional

__all__ = ['try_int', 'try_float', 'try_decimal']

NotSet = object()


def try_int(param, else_=NotSet):
    """
    >>> try_int('0')
    0
    >>> try_int('a')
    'a'
    >>> try_int('a', else_=None) is None
    True
    >>> try_int('a', else_=0)
    0
    >>> try_int(None) is None
    True
    """
    try:
        return int(param)
    except (ValueError, TypeError):
        return else_ if else_ != NotSet else param


def try_float(raw):
    try:
        return float(raw)
    except ValueError:
        return raw


def try_decimal(decimal_str: str) -> Optional[Decimal]:
    try:
        return Decimal(decimal_str)
    except InvalidOperation:
        return None



import dataclasses
from typing import TypeVar, Generic, Callable

T = TypeVar('T')

__all__ = ['Maybe', 'Some', 'Nothing']


class Maybe(Generic[T]):
    def or_else(self, maybe_callable: Callable[[], 'Maybe']):
        if isinstance(self, Some):
            return self
        else:
            return maybe_callable()

    def or_(self, maybe: 'Maybe'):
        if isinstance(self, Some):
            return self
        else:
            return maybe

    def unwrap(self):
        if isinstance(self, Some):
            return self.val
        else:
            raise ValueError('Нельзя вызвать unwrap для Nothing')


@dataclasses.dataclass(frozen=True)
class Some(Generic[T], Maybe[T]):
    val: T


class Nothing(Maybe):
    ...



__all__ = ['If', 'Then']


class _IfPart:
    def __init__(self, val):
        self.val = val() if callable(val) else val

    def then(self, other): ...

    def el(self, other): ...

    def get(self): ...

    @classmethod
    def wrap(cls, other):
        other = other() if callable(other) else other
        return other if isinstance(other, _IfPart) else cls(other)


class If(_IfPart):
    """
    >>> If(True).then('then').el('else').get()
    'then'
    >>> If(False).then('then').el('else').get()
    'else'
    """

    def then(self, other): return Then.wrap(other) if self.val else self

    def el(self, other): return Then.wrap(other)


class Then(_IfPart):
    def get(self): return self.val

    def then(self, other): return self

    def el(self, other): return self



from functools import reduce

__all__ = ['list_wo_none', 'as_list', 'unique', 'unique_by']


def list_wo_none(iter_):
    """
    >>> list_wo_none([1, 2, None])
    [1, 2]
    """
    return list(filter(None, iter_))


def as_list(item):
    """
    >>> as_list(1)
    [1]
    >>> as_list(None)
    []
    """
    if item is None:
        return []
    else:
        return [item]


def unique(list_):
    """
    >>> sorted(unique([2,2,3]))
    [2, 3]
    """
    return list(set(list_))


def unique_by(iter_, attr_):
    """
    >>> unique_by([1, 2, 2], '__str__')
    [1, 2]
    """
    return reduce(
        lambda unique_and_seen, item:
        unique_and_seen if getattr(item, attr_) in unique_and_seen[1] else
        ([*unique_and_seen[0], item], {*unique_and_seen[1], getattr(item, attr_)}),
        iter_,
        ([], set())
    )[0]



import dataclasses
from typing import TypeVar, Generic, cast, Union, Callable

__all__ = ['Either', 'Left', 'Right']

LeftT = TypeVar('LeftT')
RightT = TypeVar('RightT')


class Either(Generic[LeftT, RightT], object):
    def map_left(self, callable_):
        # type: (Callable) -> Union[Left, Right]
        if isinstance(self, Left):
            return Left(callable_(self.val))
        else:
            return cast(Right, self)

    def into_inner(self):
        # type: () -> Union[LeftT, RightT]
        if isinstance(self, Left):
            return self.val
        else:
            return cast(Right, self).val


@dataclasses.dataclass()
class Left(Either[LeftT, RightT]):
    val: LeftT


@dataclasses.dataclass()
class Right(Either[LeftT, RightT]):
    val = RightT


EitherT = TypeVar('EitherT', bound=Either)



import dataclasses
from functools import partial
from typing import Generic, Any, cast, Iterable, Tuple, Optional, TypeVar, Iterator, Callable

__all__ = ['Result', 'Ok', 'Err']

T = TypeVar('T')

OkT = TypeVar('OkT')
ErrT = TypeVar('ErrT')


class Result(Generic[OkT, ErrT]):
    def __iter__(self):
        # type: () -> Iterator[Tuple[Optional[OkT], Optional[ErrT]]]
        if isinstance(self, Ok):
            return iter(cast(Iterable[Tuple[Optional[OkT], Optional[ErrT]]], (self.value, None)))
        elif isinstance(self, Err):
            return iter(cast(Iterable[Tuple[Optional[OkT], Optional[ErrT]]], (None, self.value)))
        else:
            raise RuntimeError('Наследовать Result могут только Ok и Err')

    def map(self, any_):
        # type: (Callable[..., object]) -> Result
        """
        >>> Ok(2).map(lambda v: v + 2)
        Ok(value=4)
        >>> Err(2).map(lambda v: v + 2)
        Err(value=2)
        """
        if isinstance(self, Err):
            return self
        else:
            return Result.safe(partial(any_, cast(Ok, self).value))

    def map_err(self, any_):
        # type: (Callable[..., object]) -> Result
        """
        >>> Ok(2).map_err(lambda v: v + 2)
        Ok(value=2)
        >>> Err(2).map_err(lambda v: v + 2)
        Err(value=4)
        """
        if isinstance(self, Ok):
            return self
        else:
            return Err(any_(cast(Err, self).value))

    def or_else(self, any_):
        # type: (Callable[..., Result]) -> Result
        """
        >>> Ok(2).or_else(lambda v: Ok(v + 2))
        Ok(value=2)
        >>> Err(2).or_else(lambda v: Ok(v + 2))
        Ok(value=4)
        """
        if isinstance(self, Ok):
            return self
        else:
            return any_(cast(Err, self).value)

    def and_then(self, any_):
        # type: (Callable[..., Result]) -> Result
        """
        >>> Ok(2).and_then(lambda v: Ok(v + 2))
        Ok(value=4)
        >>> Err(2).and_then(lambda v: Ok(v + 2))
        Err(value=2)
        """
        if isinstance(self, Ok):
            return any_(cast(Ok, self).value)
        else:
            return self

    @classmethod
    def safe(cls, callable_):
        """
        >>> Result.safe(lambda: 2)
        Ok(value=2)
        >>> def raise_e():
        ...   raise Exception('err')
        >>> Result.safe(raise_e)
        Err(value=Exception('err'))
        """
        try:
            return Ok(callable_())
        except Exception as e:
            return Err(e)

    def flatten(self):
        if isinstance(self.value, Result):
            return self.value
        else:
            return self


@dataclasses.dataclass(frozen=True)
class Ok(Result):
    value: Any = None


@dataclasses.dataclass(frozen=True)
class Err(Result):
    value: Any = None



import calendar
import datetime as dt
from typing import Tuple

from dateutil.parser import parse
from dateutil.relativedelta import relativedelta

__all__ = ['end_of_day', 'end_of_month', 'previous_date_range']


def end_of_day(date_):
    """
    >>> end_of_day(dt.datetime(2022, 1, 1, 11, 11))
    datetime.datetime(2022, 1, 1, 23, 59, 59)
    """
    if not isinstance(date_, dt.datetime):
        date_ = parse(date_)

    return dt.datetime.combine(date_.date(), dt.time(23, 59, 59))


def end_of_month(start_date: dt.date) -> dt.date:
    """
    >>> end_of_month(dt.date(2019, 2, 24))
    datetime.date(2019, 2, 28)
    """
    _, end_day = calendar.monthrange(start_date.year, start_date.month)
    return dt.date(start_date.year, start_date.month, end_day)


def previous_date_range(now: dt.date = None) -> Tuple[dt.date, dt.date]:
    """
    >>> previous_date_range(dt.date(2020, 11, 5))
    (datetime.date(2020, 10, 1), datetime.date(2020, 10, 31))
    >>> previous_date_range(dt.datetime(2020, 11, 5))
    (datetime.date(2020, 10, 1), datetime.date(2020, 10, 31))
    """
    now = now or dt.datetime.now()

    from_ = (now - relativedelta(months=1)).replace(day=1)
    if isinstance(from_, dt.datetime):
        from_ = from_.date()

    to = end_of_month(from_)

    return from_, to



from decimal import Decimal, ROUND_HALF_UP
from typing import Union

__all__ = ['decimal_round']


def decimal_round(decimal_: Union[Decimal, float]) -> Decimal:
    """
    >>> decimal_round(Decimal("14.4045"))
    Decimal('14.40')
    >>> decimal_round(14.4045)
    Decimal('14.4')
    """
    if isinstance(decimal_, float):
        return Decimal(str(round(decimal_, 2)))

    return Decimal(decimal_.quantize(Decimal('.01'), rounding=ROUND_HALF_UP))



import phonenumbers

__all__ = ['is_mobile']


def is_mobile(phone: str) -> bool:
    """
    >>> is_mobile("79852489052")
    True
    >>> is_mobile('74955536678')
    False
    """
    return phonenumbers.number_type(phonenumbers.parse(phone, region="RU")) == \
           phonenumbers.PhoneNumberType.MOBILE



import operator
from functools import partial
from itertools import groupby
from typing import Iterable, TypeVar, Optional, Union, Callable, Mapping, List

__all__ = ['first', 'groupby_as_dict', 'all_eq']

T = TypeVar('T')
K = TypeVar('K')


def first(iter_):
    # type: (Iterable[T]) -> Optional[T]
    """
    >>> first([0, 1])
    0
    >>> first(range(2))
    0
    >>> first([]) is None
    True
    >>> first(None) is None
    True
    """
    if not iter_:
        return None
    return next(iter(iter_), None)


def groupby_as_dict(
    iter_,
    key_func,
    sort=False,
    flat=False,
    sort_func=None,
):
    # type: (Iterable[T], Callable[[T], K], bool, bool, Callable[[T], K]) -> Union[Mapping[K, List[T]], Mapping[K,T]]
    """
    >>> dict(groupby_as_dict([{"a": 1, "b": 2}, {"a": 2, "b": 3}], lambda obj: obj["a"]))
    {1: [{'a': 1, 'b': 2}], 2: [{'a': 2, 'b': 3}]}
    >>> dict(groupby_as_dict([{'a': 1}, {'a': 2}, {'a': 2}], lambda d: d['a']))
    {1: [{'a': 1}], 2: [{'a': 2}, {'a': 2}]}
    >>> dict(groupby_as_dict([{'a': 3}, {'a': 1}, {'a': 2}, {'a': 2}], lambda d: d['a'], sort=True))
    {1: [{'a': 1}], 2: [{'a': 2}, {'a': 2}], 3: [{'a': 3}]}
    >>> dict(groupby_as_dict([{'a': 1}, {'a': 2}, {'a': 2}], lambda d: d['a'], flat=True))
    {1: {'a': 1}, 2: {'a': 2}}
    """
    if sort or sort_func:
        sort_func = sort_func or key_func
        iter_ = sorted(iter_, key=sort_func)

    return {
        k: first(g) if flat else list(g)
        for k, g in groupby(iter_, key_func)
    }


def all_eq(iter_):
    """
    >>> all_eq(iter([3, 3, 3]))
    True
    >>> all_eq(iter([1, 1, 2]))
    False
    >>> all_eq(iter([1, 2, 1]))
    False
    >>> all_eq(iter([1, 2, 1, 1]))
    False
    """
    try:
        return all(map(partial(operator.eq, next(iter_)), iter_))
    except StopIteration:
        return True



# coding=utf-8
from operator import itemgetter

from typing import TypeVar, Tuple, Any

__all__ = ['match', 'match_str']

MatchT = TypeVar('MatchT')


def match(
    obj_to_match,  # type: MatchT
    *matchers  # type: Tuple[MatchT, Any]
):
    """
    >>> match('ass',('ass', 'tities'), ('beer', 'bitches'),)
    'tities'
    """
    for matcher, on_match in matchers:
        if matcher == obj_to_match:
            return on_match
    else:
        raise LookupError(
            u'Не нашлось совпадения: obj_to_match = {}, matchers = {}',
            obj_to_match,
            list(map(itemgetter(0), matchers)),
        )


def match_str(str_to_match, **matchers):
    """
    >>> match_str('ass',ass='tities', beer='bitches')
    'tities'
    """
    return match(str_to_match, *matchers.items())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['potyk_lib',
 'potyk_lib.dt',
 'potyk_lib.fp',
 'potyk_lib.iter_',
 'potyk_lib.num',
 'potyk_lib.phone']

package_data = \
{'': ['*']}

install_requires = \
['phonenumbers>=8.13.0,<9.0.0', 'python-dateutil>=2.8.2,<3.0.0']

setup_kwargs = {
    'name': 'potyk-lib',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'potykion',
    'author_email': 'potykion@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import cast, List, Tuple

from dateutil import parser

__all__ = ['parse_ru_date']


class RuParserInfo(parser.parserinfo):
    MONTHS = cast(
        List[Tuple[str, ...]],
        [(u"янв.", u"Январь", u'января'),
         (u"февр.", u"Февраль"),
         (u"март", u"Март"),
         (u"апр.", u"Апрель"),
         (u"май", u"Май"),
         (u"июнь", u"Июнь"),
         (u"июль", u"Июль"),
         (u"авг.", u"Август"),
         (u"сент.", u"Сентябрь",),
         (u"окт.", u"Октябрь"),
         (u"нояб.", u"Ноябрь"),
         (u"дек.", u"Декабрь", u'декабря')]
    )


def parse_ru_date(date_str):
    """
    https://stackoverflow.com/a/37485484/5500609

    >>> parse_ru_date(u'30 декабря 2021')
    datetime.date(2021, 12, 30)
    >>> parse_ru_date(u'1 января 2022')
    datetime.date(2022, 1, 1)
    """
    return parser.parse(date_str, parserinfo=RuParserInfo()).date()



import dataclasses
from typing import Callable, Tuple, Union, Any, TypeVar, Generic, Optional, Protocol

__all__ = [
    'Json', 'StatusCode', 'Response',
    'HttpRes', 'WithToJson',
]

Json = Union[dict, list]
StatusCode = Code = int
Response = Tuple[Json, StatusCode]
IdType = TypeVar('IdType')
HttpResT = TypeVar('HttpResT')


@dataclasses.dataclass
class MsgResp:
    msg: str = ''


@dataclasses.dataclass
class IdResp(Generic[IdType]):
    id: IdType


class WithToJson(Protocol):
    def to_json(self):
        ...


@dataclasses.dataclass()
class HttpRes(Generic[HttpResT]):
    success: bool
    resp: HttpResT
    code: Optional[int] = None

    def __post_init__(self):
        self.code = self.code or (200 if self.success else 400)

    @classmethod
    def ok(cls, resp=None, code=None, **kwargs):
        return cls(success=True, resp=resp or {}, code=code)

    @classmethod
    def err(cls, resp=None, code=None, **kwargs):
        return cls(success=False, resp=resp or {}, code=code)

    @classmethod
    def ok_msg(cls, msg='', code=None):
        return cls.ok(MsgResp(msg), code)

    @classmethod
    def err_msg(cls, msg='', code=None):
        return cls.err(MsgResp(msg), code)

    @classmethod
    def ok_id(cls, id_: IdType, code=None):
        return cls.ok(IdResp(id_), code)

    @classmethod
    def safe(cls, callable_: Callable[[], 'HttpRes'], ):
        try:
            return cls.ok(callable_())
        except Exception as e:
            return cls.err_msg(str(e))

    def flatten(self):
        if isinstance(self.resp, HttpRes):
            return self.resp
        else:
            return self

    def and_then(self, res: Callable[[], 'HttpRes']):
        return res() if self.success else self

    def or_else(self, res: Callable[[], 'HttpRes']):
        return self if self.success else res()

    @property
    def as_response(self) -> Response:
        return self.to_json(), self.code

    def to_json(self):
        def resp_to_json(resp):
            return (
                resp.to_json() if hasattr(resp, 'to_json') else
                (dataclasses.asdict(resp) if dataclasses.is_dataclass(resp) else
                 resp)
            )

        return (
            list(map(resp_to_json, self.resp)) if isinstance(self.resp, list) else
            resp_to_json(self.resp)
        )


def test_HttpRes():
    assert HttpRes.ok_msg('ok').as_response == ({'msg': 'ok'}, 200)
    assert HttpRes.err_msg('err').as_response == ({'msg': 'err'}, 400)
    assert HttpRes.ok_id('ok').as_response == ({'id': 'ok'}, 200)
    assert HttpRes.ok_id('ok').and_then(lambda: HttpRes.ok_msg('sam')).as_response == ({'msg': 'sam'}, 200)
    assert HttpRes.err_msg('err').and_then(lambda: HttpRes.ok_id('sam')).as_response == ({'msg': 'err'}, 400)
    assert HttpRes.ok_id('ok').or_else(lambda: HttpRes.ok_msg('sam')).as_response == ({'id': 'ok'}, 200)
    assert HttpRes.err_msg('err').or_else(lambda: HttpRes.ok_id('sam')).as_response == ({'id': 'sam'}, 200)

    class WithToJson:
        def to_json(self):
            return {'op': 'oppa'}

    assert HttpRes.ok(WithToJson()).as_response == ({'op': 'oppa'}, 200)
    assert HttpRes.ok([{'ass': 'tities'}]).as_response == ([{'ass': 'tities'}], 200)

    def raises():
        raise ValueError('op')

    assert HttpRes.safe(raises).as_response == ({'msg': 'op'}, 400)

    assert HttpRes.ok(HttpRes.ok_msg('ok')).flatten().as_response == ({'msg': 'ok'}, 200)



__all__ = ['pipe']


def pipe(*callables):
    """
    >>> sum_and_str = pipe(sum, str)
    >>> sum_and_str([1, 2])
    '3'
    """

    def _apply(val):
        for callable_ in callables:
            val = callable_(val)
        else:
            return val

    return _apply
